<?php

namespace Drupal\dpl;

use Drupal\consumers\Entity\Consumer;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;

/**
 * Provides some consumer preview links.
 */
class ConsumerPreviewLinks {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The token replacer.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $tokenReplacer;

  /**
   * ConsumerPreviewLinks constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Utility\Token $tokenReplacer
   *   The token replacer.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    Token $tokenReplacer
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->tokenReplacer = $tokenReplacer;
  }

  /**
   * A list of consumser with preview links.
   *
   * @return \Drupal\consumers\Entity\Consumer[]
   */
  public function getConsumsersWithPreviewLinks() {
    $consumer_storage = $this->entityTypeManager->getStorage('consumer');
    $consumers_with_preview_ids = $consumer_storage
      ->getQuery()
      ->condition('preview_url', '', '<>')
      ->execute();
    return $consumer_storage->loadMultiple($consumers_with_preview_ids);
  }

  /**
   * Returns a preview link for a given consumer.
   *
   * @param \Drupal\consumers\Entity\Consumer $consumer
   *   The consumer
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity we need preview links for.
   *
   * @return \Drupal\Core\Link
   */
  public function getPreviewLinkForConsumer(Consumer $consumer, EntityInterface $entity) {
    $preview_url = $consumer->get('preview_url')->value;
    $token_replaced_url = $this->tokenReplacer->replace($preview_url,
      [$entity->getEntityTypeId() => $entity]);
    return Link::fromTextAndUrl(!$consumer->get('preview_label')->isEmpty() ? $consumer->get('preview_label')->value: $this->t('Preview on @label', [
      '@label' => $consumer->label(),
    ]), Url::fromUri($token_replaced_url)->setOption('attributes', ['target' => '_blank', 'rel' => 'noopener'])
    );
  }

  public function getBrowserSizes(Consumer $consumer) {
    return array_map(function (FieldItemInterface $item) {
      $value = $item->value;
      list ($size, $label) = array_map('trim', explode('|', $value, 2));
      return new BrowserSize((int) $size, $label);
    }, iterator_to_array($consumer->get('preview_browser_sizes')));
  }

}
